@@ -48,19 +48,20 @@ CRYPTOPP_COMPILE_ASSERT(sizeof(dword) == 2*sizeof(word));
 
 #if HAVE_GCC_INIT_PRIORITY
 CRYPTOPP_COMPILE_ASSERT(CRYPTOPP_INIT_PRIORITY >= 101);
-const std::string DEFAULT_CHANNEL __attribute__ ((init_priority (CRYPTOPP_INIT_PRIORITY + 25)));
+const std::string DEFAULT_CHANNEL __attribute__ ((init_priority (CRYPTOPP_INIT_PRIORITY + 25))) = "";
 const std::string AAD_CHANNEL __attribute__ ((init_priority (CRYPTOPP_INIT_PRIORITY + 26))) = "AAD";
 const std::string &BufferedTransformation::NULL_CHANNEL = DEFAULT_CHANNEL;
 #elif HAVE_MSC_INIT_PRIORITY
 #pragma warning(disable: 4073)
 #pragma init_seg(lib)
-const std::string DEFAULT_CHANNEL;
+const std::string DEFAULT_CHANNEL = "";
 const std::string AAD_CHANNEL = "AAD";
 const std::string &BufferedTransformation::NULL_CHANNEL = DEFAULT_CHANNEL;
 #pragma warning(default: 4073)
 #else
-const std::string DEFAULT_CHANNEL;
-const std::string AAD_CHANNEL = "AAD";
+static const std::string s1(""), s2("AAD");
+const std::string DEFAULT_CHANNEL = s1;
+const std::string AAD_CHANNEL = s2;
 const std::string &BufferedTransformation::NULL_CHANNEL = DEFAULT_CHANNEL;
 #endif
 
@@ -183,7 +184,7 @@ size_t BlockTransformation::AdvancedProcessBlocks(const byte *inBlocks, const by
 	assert(inBlocks);
 	assert(outBlocks);
 	assert(length);
-	
+
 	size_t blockSize = BlockSize();
 	size_t inIncrement = (flags & (BT_InBlockIsCounter|BT_DontIncrementInOutPointers)) ? 0 : blockSize;
 	size_t xorIncrement = xorBlocks ? blockSize : 0;
@@ -261,7 +262,7 @@ void AuthenticatedSymmetricCipher::SpecifyDataLengths(lword headerLength, lword
 
 	if (messageLength > MaxMessageLength())
 		throw InvalidArgument(GetAlgorithm().AlgorithmName() + ": message length " + IntToString(messageLength) + " exceeds the maximum of " + IntToString(MaxMessageLength()));
-		
+
 	if (footerLength > MaxFooterLength())
 		throw InvalidArgument(GetAlgorithm().AlgorithmName() + ": footer length " + IntToString(footerLength) + " exceeds the maximum of " + IntToString(MaxFooterLength()));
 
@@ -317,7 +318,7 @@ word32 RandomNumberGenerator::GenerateWord32(word32 min, word32 max)
 // Stack recursion below... GenerateIntoBufferedTransformation calls GenerateBlock,
 // and GenerateBlock calls GenerateIntoBufferedTransformation. Ad infinitum. Also
 // see https://github.com/weidai11/cryptopp/issues/38.
-// 
+//
 // According to Wei, RandomNumberGenerator is an interface, and it should not
 // be instantiable. Its now spilt milk, and we are going to assert it in Debug
 // builds to alert the programmer and throw in Release builds. Developers have
@@ -368,7 +369,7 @@ public:
 	//! \brief The name of the generator
 	//! \returns the string \a NullRNGs
 	std::string AlgorithmName() const {return "NullRNG";}
-	
+
 #if defined(CRYPTOPP_DOXYGEN_PROCESSING)
 	//! \brief An implementation that throws NotImplemented
 	byte GenerateByte () {}
@@ -396,7 +397,7 @@ public:
 	void DiscardBytes (size_t n) {}
 	//! \brief An implementation that does nothing
 	void Shuffle (IT begin, IT end) {}
-	
+
 private:
 	Clonable* Clone () const { return NULL; }
 #endif
@@ -800,7 +801,7 @@ public:
 			m_ciphertext.resize(ciphertextLength);
 			m_encryptor.Encrypt(m_rng, plaintext, plaintextLength, m_ciphertext, m_parameters);
 			}
-			
+
 			FILTER_OUTPUT(1, m_ciphertext, m_ciphertext.size(), messageEnd);
 		}
 		FILTER_END_NO_MESSAGE_END;
@@ -879,7 +880,7 @@ size_t PK_Signer::SignMessage(RandomNumberGenerator &rng, const byte *message, s
 	return SignAndRestart(rng, *m, signature, false);
 }
 
-size_t PK_Signer::SignMessageWithRecovery(RandomNumberGenerator &rng, const byte *recoverableMessage, size_t recoverableMessageLength, 
+size_t PK_Signer::SignMessageWithRecovery(RandomNumberGenerator &rng, const byte *recoverableMessage, size_t recoverableMessageLength,
 	const byte *nonrecoverableMessage, size_t nonrecoverableMessageLength, byte *signature) const
 {
 	member_ptr<PK_MessageAccumulator> m(NewSignatureAccumulator(rng));
@@ -908,8 +909,8 @@ DecodingResult PK_Verifier::Recover(byte *recoveredMessage, PK_MessageAccumulato
 	return RecoverAndRestart(recoveredMessage, *m);
 }
 
-DecodingResult PK_Verifier::RecoverMessage(byte *recoveredMessage, 
-	const byte *nonrecoverableMessage, size_t nonrecoverableMessageLength, 
+DecodingResult PK_Verifier::RecoverMessage(byte *recoveredMessage,
+	const byte *nonrecoverableMessage, size_t nonrecoverableMessageLength,
 	const byte *signature, size_t signatureLength) const
 {
 	member_ptr<PK_MessageAccumulator> m(NewVerificationAccumulator());
