@@ -18,6 +18,8 @@
 #include "dsa.h"
 #include "dh.h"
 #include "mqv.h"
+#include "hmqv.h"
+#include "fhmqv.h"
 #include "luc.h"
 #include "xtrcrypt.h"
 #include "rabin.h"
@@ -46,6 +48,15 @@
 # pragma strict_gs_check (on)
 #endif
 
+// Quiet deprecated warnings intended to benefit users.
+#if CRYPTOPP_MSC_VERSION
+# pragma warning(disable: 4996)
+#endif
+
+#if CRYPTOPP_GCC_DIAGNOSTIC_AVAILABLE
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
 USING_NAMESPACE(CryptoPP)
 USING_NAMESPACE(std)
 
@@ -74,13 +85,15 @@ bool ValidateBBS()
 	bool pass = true, fail;
 	int j;
 
-	const byte output1[] = {
+	static const byte output1[] = {
 		0x49,0xEA,0x2C,0xFD,0xB0,0x10,0x64,0xA0,0xBB,0xB9,
 		0x2A,0xF1,0x01,0xDA,0xC1,0x8A,0x94,0xF7,0xB7,0xCE};
-	const byte output2[] = {
+	static const byte output2[] = {
 		0x74,0x45,0x48,0xAE,0xAC,0xB7,0x0E,0xDF,0xAF,0xD7,
 		0xD5,0x0E,0x8E,0x29,0x83,0x75,0x6B,0x27,0x46,0xA1};
 
+	// Coverity finding, also see http://stackoverflow.com/a/34509163/608639.
+	StreamState ss(cout);
 	byte buf[20];
 
 	bbs.GenerateBlock(buf, 20);
@@ -283,7 +296,7 @@ bool ValidateRSA()
 			"\x69\x94\xac\x04\xf3\x41\xb5\x7d\x05\x20\x2d\x42\x8f\xb2\xa2\x7b"
 			"\x5c\x77\xdf\xd9\xb1\x5b\xfc\x3d\x55\x93\x53\x50\x34\x10\xc1\xe1";
 
-		FileSource keys("TestData/rsa512a.dat", true, new HexDecoder);
+		FileSource keys(CRYPTOPP_DATA_DIR "TestData/rsa512a.dat", true, new HexDecoder);
 		Weak::RSASSA_PKCS1v15_MD2_Signer rsaPriv(keys);
 		Weak::RSASSA_PKCS1v15_MD2_Verifier rsaPub(rsaPriv);
 
@@ -308,7 +321,7 @@ bool ValidateRSA()
 		cout << "invalid signature verification\n";
 	}
 	{
-		FileSource keys("TestData/rsa1024.dat", true, new HexDecoder);
+		FileSource keys(CRYPTOPP_DATA_DIR "TestData/rsa1024.dat", true, new HexDecoder);
 		RSAES_PKCS1v15_Decryptor rsaPriv(keys);
 		RSAES_PKCS1v15_Encryptor rsaPub(rsaPriv);
 
@@ -335,8 +348,8 @@ bool ValidateRSA()
 		bq.Put(oaepSeed, 20);
 		FixedRNG rng(bq);
 
-		FileSource privFile("TestData/rsa400pv.dat", true, new HexDecoder);
-		FileSource pubFile("TestData/rsa400pb.dat", true, new HexDecoder);
+		FileSource privFile(CRYPTOPP_DATA_DIR "TestData/rsa400pv.dat", true, new HexDecoder);
+		FileSource pubFile(CRYPTOPP_DATA_DIR "TestData/rsa400pb.dat", true, new HexDecoder);
 		RSAES_OAEP_SHA_Decryptor rsaPriv;
 		rsaPriv.AccessKey().BERDecodePrivateKey(privFile, false, 0);
 		RSAES_OAEP_SHA_Encryptor rsaPub(pubFile);
@@ -359,7 +372,7 @@ bool ValidateDH()
 {
 	cout << "\nDH validation suite running...\n\n";
 
-	FileSource f("TestData/dh1024.dat", true, new HexDecoder());
+	FileSource f(CRYPTOPP_DATA_DIR "TestData/dh1024.dat", true, new HexDecoder());
 	DH dh(f);
 	return SimpleKeyAgreementValidate(dh);
 }
@@ -368,16 +381,250 @@ bool ValidateMQV()
 {
 	cout << "\nMQV validation suite running...\n\n";
 
-	FileSource f("TestData/mqv1024.dat", true, new HexDecoder());
+	FileSource f(CRYPTOPP_DATA_DIR "TestData/mqv1024.dat", true, new HexDecoder());
 	MQV mqv(f);
 	return AuthenticatedKeyAgreementValidate(mqv);
 }
 
+bool ValidateHMQV()
+{
+	std::cout << "\nHMQV validation suite running...\n\n";
+
+	//ECHMQV< ECP >::Domain hmqvB(false /*server*/);
+	ECHMQV256 hmqvB(false);
+	FileSource f256(CRYPTOPP_DATA_DIR "TestData/hmqv256.dat", true, new HexDecoder());
+	FileSource f384(CRYPTOPP_DATA_DIR "TestData/hmqv384.dat", true, new HexDecoder());
+	FileSource f512(CRYPTOPP_DATA_DIR "TestData/hmqv512.dat", true, new HexDecoder());
+	hmqvB.AccessGroupParameters().BERDecode(f256);
+
+	std::cout << "HMQV with NIST P-256 and SHA-256:" << std::endl;
+
+	if (hmqvB.GetCryptoParameters().Validate(GlobalRNG(), 3))
+		std::cout << "passed    authenticated key agreement domain parameters validation (server)" << std::endl;
+	else
+	{
+		std::cout << "FAILED    authenticated key agreement domain parameters invalid (server)" << std::endl;
+		return false;
+	}
+
+	const OID oid = ASN1::secp256r1();
+	ECHMQV< ECP >::Domain hmqvA(oid, true /*client*/);
+
+	if (hmqvA.GetCryptoParameters().Validate(GlobalRNG(), 3))
+		std::cout << "passed    authenticated key agreement domain parameters validation (client)" << std::endl;
+	else
+	{
+		std::cout << "FAILED    authenticated key agreement domain parameters invalid (client)" << std::endl;
+		return false;
+	}
+
+	SecByteBlock sprivA(hmqvA.StaticPrivateKeyLength()), sprivB(hmqvB.StaticPrivateKeyLength());
+	SecByteBlock eprivA(hmqvA.EphemeralPrivateKeyLength()), eprivB(hmqvB.EphemeralPrivateKeyLength());
+	SecByteBlock spubA(hmqvA.StaticPublicKeyLength()), spubB(hmqvB.StaticPublicKeyLength());
+	SecByteBlock epubA(hmqvA.EphemeralPublicKeyLength()), epubB(hmqvB.EphemeralPublicKeyLength());
+	SecByteBlock valA(hmqvA.AgreedValueLength()), valB(hmqvB.AgreedValueLength());
+
+	hmqvA.GenerateStaticKeyPair(GlobalRNG(), sprivA, spubA);
+	hmqvB.GenerateStaticKeyPair(GlobalRNG(), sprivB, spubB);
+	hmqvA.GenerateEphemeralKeyPair(GlobalRNG(), eprivA, epubA);
+	hmqvB.GenerateEphemeralKeyPair(GlobalRNG(), eprivB, epubB);
+
+	memset(valA.begin(), 0x00, valA.size());
+	memset(valB.begin(), 0x11, valB.size());
+
+	if (!(hmqvA.Agree(valA, sprivA, eprivA, spubB, epubB) && hmqvB.Agree(valB, sprivB, eprivB, spubA, epubA)))
+	{
+		std::cout << "FAILED    authenticated key agreement failed" << std::endl;
+		return false;
+	}
+
+	if (memcmp(valA.begin(), valB.begin(), hmqvA.AgreedValueLength()))
+	{
+		std::cout << "FAILED    authenticated agreed values not equal" << std::endl;
+		return false;
+	}
+
+	std::cout << "passed    authenticated key agreement" << std::endl;
+
+	// Now test HMQV with NIST P-384 curve and SHA384 hash
+	std::cout << endl;
+	std::cout << "HMQV with NIST P-384 and SHA-384:" << std::endl;
+
+	ECHMQV384 hmqvB384(false);
+	hmqvB384.AccessGroupParameters().BERDecode(f384);
+
+	if (hmqvB384.GetCryptoParameters().Validate(GlobalRNG(), 3))
+		std::cout << "passed    authenticated key agreement domain parameters validation (server)" << std::endl;
+	else
+	{
+		std::cout << "FAILED    authenticated key agreement domain parameters invalid (server)" << std::endl;
+		return false;
+	}
+
+	const OID oid384 = ASN1::secp384r1();
+	ECHMQV384 hmqvA384(oid384, true /*client*/);
+
+	if (hmqvA384.GetCryptoParameters().Validate(GlobalRNG(), 3))
+		std::cout << "passed    authenticated key agreement domain parameters validation (client)" << std::endl;
+	else
+	{
+		std::cout << "FAILED    authenticated key agreement domain parameters invalid (client)" << std::endl;
+		return false;
+	}
+
+	SecByteBlock sprivA384(hmqvA384.StaticPrivateKeyLength()), sprivB384(hmqvB384.StaticPrivateKeyLength());
+	SecByteBlock eprivA384(hmqvA384.EphemeralPrivateKeyLength()), eprivB384(hmqvB384.EphemeralPrivateKeyLength());
+	SecByteBlock spubA384(hmqvA384.StaticPublicKeyLength()), spubB384(hmqvB384.StaticPublicKeyLength());
+	SecByteBlock epubA384(hmqvA384.EphemeralPublicKeyLength()), epubB384(hmqvB384.EphemeralPublicKeyLength());
+	SecByteBlock valA384(hmqvA384.AgreedValueLength()), valB384(hmqvB384.AgreedValueLength());
+
+	hmqvA384.GenerateStaticKeyPair(GlobalRNG(), sprivA384, spubA384);
+	hmqvB384.GenerateStaticKeyPair(GlobalRNG(), sprivB384, spubB384);
+	hmqvA384.GenerateEphemeralKeyPair(GlobalRNG(), eprivA384, epubA384);
+	hmqvB384.GenerateEphemeralKeyPair(GlobalRNG(), eprivB384, epubB384);
+
+	memset(valA384.begin(), 0x00, valA384.size());
+	memset(valB384.begin(), 0x11, valB384.size());
+
+	if (!(hmqvA384.Agree(valA384, sprivA384, eprivA384, spubB384, epubB384) && hmqvB384.Agree(valB384, sprivB384, eprivB384, spubA384, epubA384)))
+	{
+		std::cout << "FAILED    authenticated key agreement failed" << std::endl;
+		return false;
+	}
+
+	if (memcmp(valA384.begin(), valB384.begin(), hmqvA384.AgreedValueLength()))
+	{
+		std::cout << "FAILED    authenticated agreed values not equal" << std::endl;
+		return false;
+	}
+
+	std::cout << "passed    authenticated key agreement" << std::endl;
+
+	return true;
+}
+
+bool ValidateFHMQV()
+{
+	std::cout << "\nFHMQV validation suite running...\n\n";
+
+	//ECFHMQV< ECP >::Domain fhmqvB(false /*server*/);
+	ECFHMQV256 fhmqvB(false);
+	FileSource f256(CRYPTOPP_DATA_DIR "TestData/fhmqv256.dat", true, new HexDecoder());
+	FileSource f384(CRYPTOPP_DATA_DIR "TestData/fhmqv384.dat", true, new HexDecoder());
+	FileSource f512(CRYPTOPP_DATA_DIR "TestData/fhmqv512.dat", true, new HexDecoder());
+	fhmqvB.AccessGroupParameters().BERDecode(f256);
+
+	std::cout << "FHMQV with NIST P-256 and SHA-256:" << std::endl;
+
+	if (fhmqvB.GetCryptoParameters().Validate(GlobalRNG(), 3))
+		std::cout << "passed    authenticated key agreement domain parameters validation (server)" << std::endl;
+	else
+	{
+		std::cout << "FAILED    authenticated key agreement domain parameters invalid (server)" << std::endl;
+		return false;
+	}
+
+	const OID oid = ASN1::secp256r1();
+	ECFHMQV< ECP >::Domain fhmqvA(oid, true /*client*/);
+
+	if (fhmqvA.GetCryptoParameters().Validate(GlobalRNG(), 3))
+		std::cout << "passed    authenticated key agreement domain parameters validation (client)" << std::endl;
+	else
+	{
+		std::cout << "FAILED    authenticated key agreement domain parameters invalid (client)" << std::endl;
+		return false;
+	}
+
+	SecByteBlock sprivA(fhmqvA.StaticPrivateKeyLength()), sprivB(fhmqvB.StaticPrivateKeyLength());
+	SecByteBlock eprivA(fhmqvA.EphemeralPrivateKeyLength()), eprivB(fhmqvB.EphemeralPrivateKeyLength());
+	SecByteBlock spubA(fhmqvA.StaticPublicKeyLength()), spubB(fhmqvB.StaticPublicKeyLength());
+	SecByteBlock epubA(fhmqvA.EphemeralPublicKeyLength()), epubB(fhmqvB.EphemeralPublicKeyLength());
+	SecByteBlock valA(fhmqvA.AgreedValueLength()), valB(fhmqvB.AgreedValueLength());
+
+	fhmqvA.GenerateStaticKeyPair(GlobalRNG(), sprivA, spubA);
+	fhmqvB.GenerateStaticKeyPair(GlobalRNG(), sprivB, spubB);
+	fhmqvA.GenerateEphemeralKeyPair(GlobalRNG(), eprivA, epubA);
+	fhmqvB.GenerateEphemeralKeyPair(GlobalRNG(), eprivB, epubB);
+
+	memset(valA.begin(), 0x00, valA.size());
+	memset(valB.begin(), 0x11, valB.size());
+
+	if (!(fhmqvA.Agree(valA, sprivA, eprivA, spubB, epubB) && fhmqvB.Agree(valB, sprivB, eprivB, spubA, epubA)))
+	{
+		std::cout << "FAILED    authenticated key agreement failed" << std::endl;
+		return false;
+	}
+
+	if (memcmp(valA.begin(), valB.begin(), fhmqvA.AgreedValueLength()))
+	{
+		std::cout << "FAILED    authenticated agreed values not equal" << std::endl;
+		return false;
+	}
+
+	std::cout << "passed    authenticated key agreement" << std::endl;
+
+	// Now test FHMQV with NIST P-384 curve and SHA384 hash
+	std::cout << endl;
+	std::cout << "FHMQV with NIST P-384 and SHA-384:" << std::endl;
+
+	ECHMQV384 fhmqvB384(false);
+	fhmqvB384.AccessGroupParameters().BERDecode(f384);
+
+	if (fhmqvB384.GetCryptoParameters().Validate(GlobalRNG(), 3))
+		std::cout << "passed    authenticated key agreement domain parameters validation (server)" << std::endl;
+	else
+	{
+		std::cout << "FAILED    authenticated key agreement domain parameters invalid (server)" << std::endl;
+		return false;
+	}
+
+	const OID oid384 = ASN1::secp384r1();
+	ECHMQV384 fhmqvA384(oid384, true /*client*/);
+
+	if (fhmqvA384.GetCryptoParameters().Validate(GlobalRNG(), 3))
+		std::cout << "passed    authenticated key agreement domain parameters validation (client)" << std::endl;
+	else
+	{
+		std::cout << "FAILED    authenticated key agreement domain parameters invalid (client)" << std::endl;
+		return false;
+	}
+
+	SecByteBlock sprivA384(fhmqvA384.StaticPrivateKeyLength()), sprivB384(fhmqvB384.StaticPrivateKeyLength());
+	SecByteBlock eprivA384(fhmqvA384.EphemeralPrivateKeyLength()), eprivB384(fhmqvB384.EphemeralPrivateKeyLength());
+	SecByteBlock spubA384(fhmqvA384.StaticPublicKeyLength()), spubB384(fhmqvB384.StaticPublicKeyLength());
+	SecByteBlock epubA384(fhmqvA384.EphemeralPublicKeyLength()), epubB384(fhmqvB384.EphemeralPublicKeyLength());
+	SecByteBlock valA384(fhmqvA384.AgreedValueLength()), valB384(fhmqvB384.AgreedValueLength());
+
+	fhmqvA384.GenerateStaticKeyPair(GlobalRNG(), sprivA384, spubA384);
+	fhmqvB384.GenerateStaticKeyPair(GlobalRNG(), sprivB384, spubB384);
+	fhmqvA384.GenerateEphemeralKeyPair(GlobalRNG(), eprivA384, epubA384);
+	fhmqvB384.GenerateEphemeralKeyPair(GlobalRNG(), eprivB384, epubB384);
+
+	memset(valA384.begin(), 0x00, valA384.size());
+	memset(valB384.begin(), 0x11, valB384.size());
+
+	if (!(fhmqvA384.Agree(valA384, sprivA384, eprivA384, spubB384, epubB384) && fhmqvB384.Agree(valB384, sprivB384, eprivB384, spubA384, epubA384)))
+	{
+		std::cout << "FAILED    authenticated key agreement failed" << std::endl;
+		return false;
+	}
+
+	if (memcmp(valA384.begin(), valB384.begin(), fhmqvA384.AgreedValueLength()))
+	{
+		std::cout << "FAILED    authenticated agreed values not equal" << std::endl;
+		return false;
+	}
+
+	std::cout << "passed    authenticated key agreement" << std::endl;
+
+	return true;
+}
+
 bool ValidateLUC_DH()
 {
 	cout << "\nLUC-DH validation suite running...\n\n";
 
-	FileSource f("TestData/lucd512.dat", true, new HexDecoder());
+	FileSource f(CRYPTOPP_DATA_DIR "TestData/lucd512.dat", true, new HexDecoder());
 	LUC_DH dh(f);
 	return SimpleKeyAgreementValidate(dh);
 }
@@ -386,7 +633,7 @@ bool ValidateXTR_DH()
 {
 	cout << "\nXTR-DH validation suite running...\n\n";
 
-	FileSource f("TestData/xtrdh171.dat", true, new HexDecoder());
+	FileSource f(CRYPTOPP_DATA_DIR "TestData/xtrdh171.dat", true, new HexDecoder());
 	XTR_DH dh(f);
 	return SimpleKeyAgreementValidate(dh);
 }
@@ -396,7 +643,7 @@ bool ValidateElGamal()
 	cout << "\nElGamal validation suite running...\n\n";
 	bool pass = true;
 	{
-		FileSource fc("TestData/elgc1024.dat", true, new HexDecoder);
+		FileSource fc(CRYPTOPP_DATA_DIR "TestData/elgc1024.dat", true, new HexDecoder);
 		ElGamalDecryptor privC(fc);
 		ElGamalEncryptor pubC(privC);
 		privC.AccessKey().Precompute();
@@ -414,7 +661,7 @@ bool ValidateDLIES()
 	cout << "\nDLIES validation suite running...\n\n";
 	bool pass = true;
 	{
-		FileSource fc("TestData/dlie1024.dat", true, new HexDecoder);
+		FileSource fc(CRYPTOPP_DATA_DIR "TestData/dlie1024.dat", true, new HexDecoder);
 		DLIES<>::Decryptor privC(fc);
 		DLIES<>::Encryptor pubC(privC);
 		pass = CryptoSystemValidate(privC, pubC) && pass;
@@ -437,7 +684,7 @@ bool ValidateNR()
 	cout << "\nNR validation suite running...\n\n";
 	bool pass = true;
 	{
-		FileSource f("TestData/nr2048.dat", true, new HexDecoder);
+		FileSource f(CRYPTOPP_DATA_DIR "TestData/nr2048.dat", true, new HexDecoder);
 		NR<SHA>::Signer privS(f);
 		privS.AccessKey().Precompute();
 		NR<SHA>::Verifier pubS(privS);
@@ -459,14 +706,14 @@ bool ValidateDSA(bool thorough)
 	cout << "\nDSA validation suite running...\n\n";
 
 	bool pass = true;
-	FileSource fs1("TestData/dsa1024.dat", true, new HexDecoder());
+	FileSource fs1(CRYPTOPP_DATA_DIR "TestData/dsa1024.dat", true, new HexDecoder());
 	DSA::Signer priv(fs1);
 	DSA::Verifier pub(priv);
-	FileSource fs2("TestData/dsa1024b.dat", true, new HexDecoder());
+	FileSource fs2(CRYPTOPP_DATA_DIR "TestData/dsa1024b.dat", true, new HexDecoder());
 	DSA::Verifier pub1(fs2);
 	assert(pub.GetKey() == pub1.GetKey());
 	pass = SignatureValidate(priv, pub, thorough) && pass;
-	pass = RunTestDataFile("TestVectors/dsa.txt", g_nullNameValuePairs, thorough) && pass;
+	pass = RunTestDataFile(CRYPTOPP_DATA_DIR "TestVectors/dsa.txt", g_nullNameValuePairs, thorough) && pass;
 
 	return pass;
 }
@@ -477,7 +724,7 @@ bool ValidateLUC()
 	bool pass=true;
 
 	{
-		FileSource f("TestData/luc1024.dat", true, new HexDecoder);
+		FileSource f(CRYPTOPP_DATA_DIR "TestData/luc1024.dat", true, new HexDecoder);
 		LUCSSA_PKCS1v15_SHA_Signer priv(f);
 		LUCSSA_PKCS1v15_SHA_Verifier pub(priv);
 		pass = SignatureValidate(priv, pub) && pass;
@@ -494,14 +741,14 @@ bool ValidateLUC_DL()
 {
 	cout << "\nLUC-HMP validation suite running...\n\n";
 
-	FileSource f("TestData/lucs512.dat", true, new HexDecoder);
+	FileSource f(CRYPTOPP_DATA_DIR "TestData/lucs512.dat", true, new HexDecoder);
 	LUC_HMP<SHA>::Signer privS(f);
 	LUC_HMP<SHA>::Verifier pubS(privS);
 	bool pass = SignatureValidate(privS, pubS);
 
 	cout << "\nLUC-IES validation suite running...\n\n";
 
-	FileSource fc("TestData/lucc512.dat", true, new HexDecoder);
+	FileSource fc(CRYPTOPP_DATA_DIR "TestData/lucc512.dat", true, new HexDecoder);
 	LUC_IES<>::Decryptor privC(fc);
 	LUC_IES<>::Encryptor pubC(privC);
 	pass = CryptoSystemValidate(privC, pubC) && pass;
@@ -515,7 +762,7 @@ bool ValidateRabin()
 	bool pass=true;
 
 	{
-		FileSource f("TestData/rabi1024.dat", true, new HexDecoder);
+		FileSource f(CRYPTOPP_DATA_DIR "TestData/rabi1024.dat", true, new HexDecoder);
 		RabinSS<PSSR, SHA>::Signer priv(f);
 		RabinSS<PSSR, SHA>::Verifier pub(priv);
 		pass = SignatureValidate(priv, pub) && pass;
@@ -532,7 +779,7 @@ bool ValidateRW()
 {
 	cout << "\nRW validation suite running...\n\n";
 
-	FileSource f("TestData/rw1024.dat", true, new HexDecoder);
+	FileSource f(CRYPTOPP_DATA_DIR "TestData/rw1024.dat", true, new HexDecoder);
 	RWSS<PSSR, SHA>::Signer priv(f);
 	RWSS<PSSR, SHA>::Verifier pub(priv);
 
@@ -544,7 +791,7 @@ bool ValidateBlumGoldwasser()
 {
 	cout << "\nBlumGoldwasser validation suite running...\n\n";
 
-	FileSource f("TestData/blum512.dat", true, new HexDecoder);
+	FileSource f(CRYPTOPP_DATA_DIR "TestData/blum512.dat", true, new HexDecoder);
 	BlumGoldwasserPrivateKey priv(f);
 	BlumGoldwasserPublicKey pub(priv);
 
@@ -559,7 +806,7 @@ bool TestPolynomialMod2()
 	bool pass1 = true, pass2 = true, pass3 = true;
 
 	cout << "\nTesting PolynomialMod2 bit operations...\n\n";
-	
+
 	static const unsigned int start = 0;
 	static const unsigned int stop = 4 * WORD_BITS + 1;
 
@@ -567,56 +814,56 @@ bool TestPolynomialMod2()
 	{
 		PolynomialMod2 p(1);
 		p <<= i;
-		
+
 		Integer n(Integer::One());
 		n <<= i;
-		
+
 		std::ostringstream oss1;
 		oss1 << p;
-		
+
 		std::string str1, str2;
-		
+
 		// str1 needs the commas removed used for grouping
 		str1 = oss1.str();
 		str1.erase(std::remove(str1.begin(), str1.end(), ','), str1.end());
-		
+
 		// str1 needs the trailing 'b' removed
 		str1.erase(str1.end() - 1);
 
 		// str2 is fine as-is
 		str2 = IntToString(n, 2);
-		
+
 		pass1 &= (str1 == str2);
 	}
-	
+
 	for (unsigned int i=start; i < stop; i++)
 	{
-		const word w(SIZE_MAX);
+		const word w((word)SIZE_MAX);
 
 		PolynomialMod2 p(w);
 		p <<= i;
 
 		Integer n(Integer::POSITIVE, static_cast<lword>(w));
 		n <<= i;
-		
+
 		std::ostringstream oss1;
 		oss1 << p;
-		
+
 		std::string str1, str2;
-		
+
 		// str1 needs the commas removed used for grouping
 		str1 = oss1.str();
 		str1.erase(std::remove(str1.begin(), str1.end(), ','), str1.end());
-		
+
 		// str1 needs the trailing 'b' removed
 		str1.erase(str1.end() - 1);
 
 		// str2 is fine as-is
 		str2 = IntToString(n, 2);
-		
+
 		pass2 &= (str1 == str2);
 	}
-	
+
 	RandomNumberGenerator& prng = GlobalRNG();
 	for (unsigned int i=start; i < stop; i++)
 	{
@@ -628,16 +875,16 @@ bool TestPolynomialMod2()
 
 		Integer n(Integer::POSITIVE, static_cast<lword>(w));
 		n <<= i;
-		
+
 		std::ostringstream oss1;
 		oss1 << p;
-		
+
 		std::string str1, str2;
-		
+
 		// str1 needs the commas removed used for grouping
 		str1 = oss1.str();
 		str1.erase(std::remove(str1.begin(), str1.end(), ','), str1.end());
-		
+
 		// str1 needs the trailing 'b' removed
 		str1.erase(str1.end() - 1);
 
@@ -651,13 +898,13 @@ bool TestPolynomialMod2()
 			cout << "     str1: " << str1 << "\n";
 			cout << "     str2: " << str2 << "\n";
 		}
-		
+
 		pass3 &= (str1 == str2);
 	}
 
-	cout << (!pass1 ? "FAILED" : "passed") << "    " << "1 shifted over range [" << dec << start << "," << stop << "]" << "\n";
-	cout << (!pass2 ? "FAILED" : "passed") << "    " << "0x" << hex << word(SIZE_MAX) << dec << " shifted over range [" << start << "," << stop << "]" << "\n";
-	cout << (!pass3 ? "FAILED" : "passed") << "    " << "random values shifted over range [" << dec << start << "," << stop << "]" << "\n";
+	cout << (!pass1 ? "FAILED" : "passed") << ":  " << "1 shifted over range [" << dec << start << "," << stop << "]" << "\n";
+	cout << (!pass2 ? "FAILED" : "passed") << ":  " << "0x" << hex << word(SIZE_MAX) << dec << " shifted over range [" << start << "," << stop << "]" << "\n";
+	cout << (!pass3 ? "FAILED" : "passed") << ":  " << "random values shifted over range [" << dec << start << "," << stop << "]" << "\n";
 
 	if (!(pass1 && pass2 && pass3))
 		cout.flush();
@@ -828,7 +1075,7 @@ bool ValidateESIGN()
 		"\x74\x02\x37\x0E\xED\x0A\x06\xAD\xF4\x15\x65\xB8\xE1\xD1\x45\xAE\x39\x19\xB4\xFF\x5D\xF1\x45\x7B\xE0\xFE\x72\xED\x11\x92\x8F\x61\x41\x4F\x02\x00\xF2\x76\x6F\x7C"
 		"\x79\xA2\xE5\x52\x20\x5D\x97\x5E\xFE\x39\xAE\x21\x10\xFB\x35\xF4\x80\x81\x41\x13\xDD\xE8\x5F\xCA\x1E\x4F\xF8\x9B\xB2\x68\xFB\x28";
 
-	FileSource keys("TestData/esig1536.dat", true, new HexDecoder);
+	FileSource keys(CRYPTOPP_DATA_DIR "TestData/esig1536.dat", true, new HexDecoder);
 	ESIGN<SHA>::Signer signer(keys);
 	ESIGN<SHA>::Verifier verifier(signer);
 
